package com.demo;

import io.grpc.*;
import io.opentelemetry.api.trace.Span;
import io.opentelemetry.api.trace.Tracer;
import io.opentelemetry.exporter.jaeger.JaegerGrpcSpanExporter;
import io.opentelemetry.sdk.OpenTelemetrySdk;
import io.opentelemetry.sdk.trace.export.SimpleSpanProcessor;

public class OpenTelemetryExample {
    // OTel API
    private Tracer tracer = null;


    private void initTracer(String name, int port, String auth) {
        // Create a channel towards Jaeger end point
        ManagedChannel jaegerChannel = ManagedChannelBuilder.forAddress(name, port).intercept(new ClientInterceptor() {
            @Override
            public <ReqT, RespT> ClientCall<ReqT, RespT> interceptCall(MethodDescriptor<ReqT, RespT> methodDescriptor, CallOptions callOptions, Channel channel) {
                return new ForwardingClientCall.SimpleForwardingClientCall<ReqT, RespT>(
                        channel.newCall(methodDescriptor, callOptions)) {
                    @Override
                    public void start(Listener<RespT> responseListener, final Metadata headers) {
                        Metadata.Key<String> headerKey = Metadata.Key.of("Authentication", Metadata.ASCII_STRING_MARSHALLER);
                        headers.put(headerKey, auth);
                        super.start(responseListener, headers);
                    }
                };
            }
        }).usePlaintext().build();

        // Export traces to Jaeger
        JaegerGrpcSpanExporter jaegerExporter =
                JaegerGrpcSpanExporter.builder()
                        .setServiceName("otel-jaeger-example")
                        .setChannel(jaegerChannel)
                        .setDeadlineMs(30000)
                        .build();

        // Set to process the spans by the Jaeger Exporter
        OpenTelemetrySdk openTelemetry = OpenTelemetrySdk.builder().build();
        openTelemetry
                .getTracerManagement()
                .addSpanProcessor(SimpleSpanProcessor.builder(jaegerExporter).build());


        openTelemetry.getTracerManagement();
        tracer = openTelemetry.getTracer("io.opentelemetry.example.JaegerExample");

    }

    private void myWonderfulUseCase() {
        // Generate a span
        Span span = this.tracer.spanBuilder("Start my wonderful use case").startSpan();
        span.addEvent("Event 0");
        // execute my use case - here we simulate a wait
        doWork();
        span.addEvent("Event 1");
        span.end();
    }

    private void doWork() {
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
        }
    }

    public static void main(String[] args) {

        OpenTelemetryExample example = new OpenTelemetryExample();
        // please replease token which you can get from "http://tracing.console.aliyun.com/"
        example.initTracer("tracing-analysis-dc-bj.aliyuncs.com", 1883, "123@0yourToken_123@0yourToken");
        example.myWonderfulUseCase();
        // wait some seconds
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
        }
        System.out.println("Bye");
    }
}
